# This migration tool converts OSQA data to Askbot, optimized for big databases
# Copyright (C) 2016  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from __future__ import print_function

from . import django_integration
from . import utils


class Migrator:

    def __init__(self, osqa_conn, askbot_path, askbot_env):
        # Connect to the OSQA database
        self.osqa = utils.Connection(osqa_conn)

        # Load Askbot
        self.django = django_integration.load_django(askbot_path, askbot_env)
        self.models = django_integration.load_models()
        self.openid_models = django_integration.load_openid_models()

    def set_next_id(self, table, field, value):
        """Set the next ID for a sequence"""
        self.askbot_query(
            "SELECT setval("
                "(SELECT pg_get_serial_sequence(%s, %s)), %s"
            ")"
        ";", table, field, value)

    def askbot_query(self, query, *params):
        """Execute a query against Askbot"""
        with self.django.db.connection.cursor() as cursor:
            cursor.execute(query, params)
            return cursor.fetchall()

    def migrate(self):
        """Start the migration process"""
        self.migrate_users()
        self.migrate_user_logins()

    def migrate_users(self):
        """Migrate users to Askbot"""
        migrate_fields_user = [
            "id", "email", "username", "first_name", "last_name", "last_login",
            "date_joined",
        ]
        migrate_fields_profile = [
            "website", "email_isvalid", "real_name", "date_of_birth",
            "location", "last_seen",
        ]

        # Load existing users
        existing = [u[0] for u in self.askbot_query(
            "SELECT id FROM auth_user;"
        )]

        last_id = 0
        tbl = "auth_user a, forum_user f WHERE a.id = f.user_ptr_id"
        for user in self.osqa.table_items(tbl, what="users", order="id"):
            last_id = user["id"]

            # Check if the user already exists
            if user["id"] in existing:
                continue

            # Create a new User object
            new = self.models.User()

            # Copy some fields
            for field in migrate_fields_user:
                setattr(new, field, user[field])

            # Update some other details of the user
            new.set_unusable_password()
            if user["is_superuser"]:
                new.set_admin_status()

            # Save the user
            new.save()

            # Update the user profile
            for field in migrate_fields_profile:
                setattr(new, field, user[field])

            # Update the about text
            new.update_localized_profile(about=user["about"])

            # Save the user again
            new.save()

        # Update the next ID value
        self.set_next_id("auth_user", "id", last_id + 1)

    def migrate_user_logins(self):
        """Migrate the login table"""
        # Load existing users
        existing = [u[0] for u in self.askbot_query(
            "SELECT id FROM django_authopenid_userassociation;"
        )]

        last_id = 0
        t = "forum_authkeyuserassociation"
        for login in self.osqa.table_items(t, what="user logins", order="id"):
            last_id = login["id"]

            # Check if the login entry already exists
            if login["id"] in existing:
                continue

            # Migrate login data
            assoc = self.openid_models.UserAssociation()
            assoc.id = login["id"]
            assoc.openid_url = login["key"]
            assoc.user_id = login["user_id"]
            assoc.provider_name = login["provider"]
            assoc.last_used_timestamp = login["added_at"]
            try:
                assoc.save()
            except self.django.db.utils.IntegrityError:
                # OSQA forgot to delete a record -- don't add it
                pass

        # Update the next ID value
        self.set_next_id("django_authopenid_userassociation", "id", last_id + 1)
