# This migration tool converts OSQA data to Askbot, optimized for big databases
# Copyright (C) 2016  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from __future__ import print_function

import sys

import psycopg2


class ProgressBar:

    def __init__(self, action, items, total):
        self.action = action
        self.items = items
        self.current = 0
        self.total = total

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *_):
        self.stop()

    def start(self):
        self.to(0)

    def stop(self):
        self.repaint()
        print()

    def to(self, current):
        """Set the current value of the progress bar"""
        self.current = current
        self.repaint()

    def repaint(self):
        """Repaint the progress bar"""
        if self.current != self.total:
            msg = "%s: %s/%s" % (self.action, self.current, self.total)
        else:
            msg = "%s: completed (%s %s)" % (
                    self.action, self.total, self.items
            ),

        sys.stdout.write("\r%s" % msg)
        sys.stdout.flush()


class Connection:

    def __init__(self, connstring):
        self.conn = psycopg2.connect(connstring)

    def query(self, query, *params):
        """Execute a query against the DB"""
        cursor = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        cursor.execute(query, params)
        return cursor.fetchall()

    def table_items(self, name, batch=100, what=None, order=None):
        """Return items in a table without loading all of them in memory"""
        # This loads elements in batches, to avoid loading too much stuff in
        # memory and going OOM
        total = self.query("SELECT COUNT(*) FROM %s;" % name)[0][0]

        if order is not None:
            order = "ORDER BY %s" % order
        else:
            order = ""

        pb = None
        if what is not None:
            pb = ProgressBar("Migrating %s" % what, what, total)
            pb.start()

        steps = total / batch
        if total % batch != 0:
            steps += 1

        for i in range(steps):
            query = (
                "SELECT * FROM %s %s LIMIT %s OFFSET %s;" %
                (name, order, batch, i * batch)
            )
            for (i2, item) in enumerate(self.query(query)):
                if pb is not None:
                    pb.to((i * batch) + i2 + 1)

                yield item

        if pb:
            pb.stop()
